/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file Append.cpp
 * Implementation file for the implementations of the Append helper methods.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include "Append.h"
#include "Convert.h"


///////////////////////////////////////////////////////////////////////////////
// namespace
///////////////////////////////////////////////////////////////////////////////

using namespace std;


///////////////////////////////////////////////////////////////////////////////
// typedefs
///////////////////////////////////////////////////////////////////////////////

typedef std::vector<unsigned char> CByteVec;


/**
 * This will append an unsigned int value to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param nValue   - The value to append
 */
void Append( CByteVec& vecData, unsigned int nValue )
{
	// Swap the byte order
	const unsigned int nSwapped = CConvert::Swap32( nValue );

	// Now append it as a byte array
	Append( vecData, (char*)&nSwapped, sizeof(nSwapped));
}


/**
 * This will append an unsigned short value to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param nValue   - The value to append
 */
void Append( CByteVec& vecData, unsigned short nValue)
{
	// Swap the byte order
	const unsigned short nSwapped = CConvert::Swap16( nValue );

	// Now append it as a byte array
	Append( vecData, (char*)&nSwapped, sizeof(nSwapped));
}


/**
 * This will append an unsigned character buffer to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param pcValue - The data to append.
 * @param nValueLength - The length of the data to append
 */
void Append( CByteVec& vecData, const unsigned char* pcValue, size_t nValueLength )
{
	// Prevent the vector from having to expand during copy, while not forcing it to shrink
	vecData.reserve( vecData.size() + nValueLength );

	// Insert pcValue to the end of vecData
	vecData.insert( vecData.end(), pcValue, pcValue + nValueLength );
}


/**
 * This will append a char buffer to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param pcValue - The data to append.
 * @param nValueLength - The length of the data to append
 */
void Append( CByteVec& vecData, const char* pcValue, size_t nValueLength )
{
	// Prevent the vector from having to expand during copy, while not forcing it to shrink
	vecData.reserve( vecData.size() + nValueLength );

	// Insert pcValue to the end of vecData
	vecData.insert( vecData.end(), pcValue, pcValue + nValueLength );
}


/**
 * This will append a std::string to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param pcValue - The data to append.
 * @param nValueLength - The length of the data to append
 */
void Append( CByteVec& vecData, const std::string& strValue )
{
	// Prevent the vector from having to expand during copy, while not forcing it to shrink
	vecData.reserve( vecData.size() + strValue.size());

	// Insert strValue to the end of vecData
	vecData.insert( vecData.end(), strValue.begin(), strValue.end());
}


/**
 * This will append a std::string to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param strValue - The data to append.
 * @param nFieldSize - The length of the data to append
 * @param cFill - A character that will fill any leftover space
 */
void AppendFixedLen( CByteVec& vecData, const std::string& strValue, size_t nFieldSize, char cFill )
{
	// Prevent the vector from having to expand during copy, while not forcing it to shrink
	vecData.reserve( vecData.size() + nFieldSize );

	// Do we need to use temporary storage for the copy?
	if( strValue.size() == nFieldSize )
	{
		// No temporary needed, copy straight from input
		vecData.insert( vecData.end(), strValue.begin(), strValue.end());
	}
	else
	{
		// Need to make a temp string with the right length
		std::string strTemp( strValue );

		// Resize the string to the specified size passed in and make sure
		// to fill the extra space with the character passed in.
		strTemp.resize( nFieldSize, cFill );

		// Insert strValue to the end of vecData
		vecData.insert( vecData.end(), strTemp.begin(), strTemp.end());
	}
}


/**
 * This will append a vector of bits to the passed in vector data
 *
 * @param vecOut - The vector to append the data to.
 * @param vecBitsIn - The vector from which the data will be used for the append operation.
 */
void Append( CByteVec& vecOut, const vector<bool>& vecBitsIn )
{
	unsigned char cByte = 0;
	unsigned char cMask = 0x80; // (10000000 binary)

	// Iterate through the source vector.
	for( vector<bool>::const_iterator pBit = vecBitsIn.begin(); pBit != vecBitsIn.end(); ++pBit )
	{
		cByte |= *pBit ? cMask : 0;

		cMask >>= 1;

		if( 0 == cMask )
		{
			vecOut.push_back( cByte );

			cByte = 0;

			cMask = 0x80;
		}
	}

	if( 0x80 != cMask  ) //we've got a partial byte, possibly of all zeros
	{
		vecOut.push_back( cByte );
	}
}

