/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file
 * Implementation file for helper methods that convert object from one type
 * to another.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include "Convert.h"
#include "StringUtils.h"
#include "ApexException.h"
#include <sstream>
#include <memory>

#ifdef UNDER_CE
#include <windows.h>
#endif

///////////////////////////////////////////////////////////////////////////////
// namespaces
///////////////////////////////////////////////////////////////////////////////

using namespace std;


/** 
 * Converts a value into a multibyte string.
 *
 * @param nValue A size_t value to be converted.
 *
 * @return A std::string based on the input value.
 */
string CConvert::IntToStrA(size_t nValue)
{
	ostringstream o;

	o << nValue;

	return o.str();
}


/** 
 * Converts a value into a multibyte string.
 *
 * @param nValue An integer value to be converted.
 *
 * @return A std::string based on the input value.
 */
string CConvert::IntToStrA(int nValue)
{
	ostringstream o;

	o << nValue;

	return o.str();
}


/** 
 * Converts a character into a multibyte string.
 *
 * @param cValue A char to be converted to a string.
 *
 * @return std::string based on the input value.
 */
string CConvert::CharToStrA( char cValue )
{
	return string( 1, cValue );
}


/** 
 * Converts an unsigned character into a multibyte string.
 *
 * @param cValue An unsigned char to be converted to a string.
 *
 * @return std::string based on the input value.
 */
string CConvert::UCharToStrA(unsigned char cValue)
{
	return string( 1, static_cast<char>( cValue ));
}


/** 
 * Swap the byte order for a short value.
 *
 * @param nNum A short value to be swapped.
 *
 * @return The swapped value.
 */
short CConvert::Swap16(short nNum)
{
    return (((nNum >> 8) & 0xFF) | ((nNum << 8) & 0xFF00));
}


/** 
 * Swap the byte order for an unsigned short value.
 *
 * @param nNum An unsigned short value to be swapped.
 *
 * @return An unsigned short value that is the swapped value.
 */
unsigned short CConvert::Swap16(unsigned short nNum)
{
    return (((nNum >> 8) & 0xFF) | ((nNum << 8) & 0xFF00));
}


/** 
 * Swap the byte order for a int value.
 *
 * @param nNum An integer value to be swapped.
 *
 * @return An integer value that is the swapped value.
 */
int CConvert::Swap32(int nNum)
{	
	int nReturn = 0;

	nReturn = Swap32((unsigned int)nNum);

	return nReturn;
}


/** 
 * Swap the byte order for an unsigned int value.
 *
 * @param nNum An unsigned integer value to be swapped.
 *
 * @return An unsigned integer value that is the swapped value.
 */
unsigned int CConvert::Swap32(unsigned int nNum)
{	
	unsigned int nReturn = 0;

	nReturn = (((nNum >> 24) & 0xFF) | ((nNum >> 8) & 0xFF00) | \
               ((nNum << 8) & 0xFF0000) | ((nNum << 24) & 0xFF000000));

	return nReturn;
}


/** 
 * Swap the byte order for a long value.
 *
 * @param nNum A long value to be swapped.
 *
 * @return A long value that is the swapped value.
 */
long CConvert::Swap32(long nNum)
{	
	long nReturn = 0;

	// calculate
	nReturn =  static_cast<long>(Swap32((unsigned long)nNum));

	return nReturn;
}


/** 
 * Swap the byte order for an unsigned long value.
 *
 * @param nNum An unsigned long value to be swapped.
 *
 * @return An unsigned long value that is the swapped value.
 */
unsigned long CConvert::Swap32(unsigned long nNum)
{	
	unsigned long nReturn = 0;

	nReturn = (((nNum >> 24) & 0xFF) | ((nNum >> 8) & 0xFF00) | \
               ((nNum << 8) & 0xFF0000) | ((nNum << 24) & 0xFF000000));

	return nReturn;
}


/**
 * This will convert a hexidecimal string in readable format to a vector of unsigned characters.
 *
 * @param pcIn - unsigned char* that is the pointer to the input data. 
 * @param nInLen - is the number of characters in the input data to convert.
 *
 * @return A hexidecial string representation of the binary data.
 */
string CConvert::BinToHexStringA(const unsigned char* pcIn, size_t nInLength)
{
	string strText = "";

	try
	{
		unsigned char* encoding = (unsigned char*)"0123456789abcdef";
		size_t nOutHexLen = nInLength*2;
		std::auto_ptr<unsigned char> pcOutHex(new unsigned char[nOutHexLen+1]);
		memset(pcOutHex.get(), '\0', nOutHexLen+1);

		size_t i = 0;
		size_t j = 0;
		unsigned char c = '\0';
		unsigned char clow = '\0';
		unsigned char cTwo = '\0';

		for (i=0; i < nInLength; i++)
		{
			c = ((unsigned char)pcIn[i]);
			clow = (unsigned char)(c & 15); // get the lower nibble
			cTwo = encoding[clow];
			clow = (unsigned char)(c >> 4); // get higher nibble
			pcOutHex.get()[j++] = encoding[clow];
			pcOutHex.get()[j++] = cTwo;
		}

		strText = (char*)(pcOutHex.get());
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		string strErr = "Error occured in CConvert::BinToHexString().";
		throw CApexException( ERROR_FATAL, strErr, __FILE__, __LINE__);
	}
	return strText;
}


/**
 * This will convert a string that is that is in readable hex format into binary.
 *
 * @param str - const string& str that is the input hexidicimal string.
 * @param pcOut - unsigned char* that is the pointer in which to put the data. Note that this must be allocated
 *                prior to passing into this function.  The length sould be 1/2 of the string input length.
 * @param nOutLen - is the number of characters in the output buffer string. Upon return this will set the
 *                  number of binary digits.
 *
 * @note It will throw and error if the string is of 0 length.
 */
void CConvert::HexStringToBin(const std::string& str, unsigned char* pcOut, size_t& nOutLength)
{
	try
	{
		size_t len = (int)str.length();
		if (str.empty())
		{
			string strErr = "The input lenght can not be 0.";
			throw CApexException( ERROR_FATAL, strErr, __FILE__, __LINE__);
		}
		
		if (len/2 > nOutLength)
		{
			string strErr = "The output buffer is too small.";
			throw CApexException( ERROR_FATAL, strErr, __FILE__, __LINE__);
		}

		// check that the input length can be devided by 2
		bool bResult = CheckInputDiv(len, 2);
		if (bResult == false)
		{
			string strErr = "The hexidecimal input was not devisable by 2.";
			throw CApexException( ERROR_FATAL, strErr, __FILE__, __LINE__);
		}


		for (int i = 0; i < (int)(len / 2); ++i)
		{
			string strHex = str.substr(i*2, 2);
			char* pcIn = (char*)strHex.c_str();
			char* pc = (char*)strHex.c_str();
			unsigned char uc = (unsigned char)strtoul(pcIn, &pc, 16);
			pcOut[i] = (uc);
		}

		// set the out length
		nOutLength = len/2;
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		string strErr = "Unknown error occured in CConvert::HexStringToBin().";
		throw CApexException( ERROR_FATAL, strErr, __FILE__, __LINE__);
	}
}


string CConvert::IntToHexStringA(int nNum)
{
	string strText = "0x";
	const unsigned int nNumSize = sizeof(int);
	auto_ptr<char> pstrHex(new char[2*nNumSize + 1]);

	for (int i = 2*nNumSize - 1, j = 0; i >= 0; i--, j++)
	{
		pstrHex.get()[j] = "0123456789ABCDEF"[((nNum >> i*4) & 0xF)];
	}
	pstrHex.get()[2*nNumSize] = '\0';

	strText.append(pstrHex.get());

	return strText;
}


string CConvert::IntToHexStringA(unsigned int nNum)
{
	string strText = "0x";
	const unsigned int nNumSize = sizeof(unsigned int);
	auto_ptr<char> pstrHex(new char[2*nNumSize + 1]);

	for (int i = 2*nNumSize - 1, j = 0; i >= 0; i--, j++)
	{
		pstrHex.get()[j] = "0123456789ABCDEF"[((nNum >> i*4) & 0xF)];
	}
	pstrHex.get()[2*nNumSize] = '\0';

	strText.append(pstrHex.get());

	return strText;
}


string CConvert::ShortToHexStringA(short nNum)
{
	string strText = "0x";
	const unsigned int nNumSize = sizeof(short);
	auto_ptr<char> pstrHex(new char[2*nNumSize + 1]);

	for (int i = 2*nNumSize - 1, j = 0; i >= 0; i--, j++)
	{
		pstrHex.get()[j] = "0123456789ABCDEF"[((nNum >> i*4) & 0xF)];
	}
	pstrHex.get()[2*nNumSize] = '\0';

	strText.append(pstrHex.get());

	return strText;
}


string CConvert::ShortToHexStringA(unsigned short nNum)
{
	string strText = "0x";
	const unsigned int nNumSize = sizeof(unsigned short);
	auto_ptr<char> pstrHex(new char[2*nNumSize + 1]);

	for (int i = 2*nNumSize - 1, j = 0; i >= 0; i--, j++)
	{
		pstrHex.get()[j] = "0123456789ABCDEF"[((nNum >> i*4) & 0xF)];
	}
	pstrHex.get()[2*nNumSize] = '\0';

	strText.append(pstrHex.get());

	return strText;
}


/**
 * This checks that an integer is divisable by a number.
 *
 * @param nLen - int to test if it is devisable by nBase.
 * @param nBase - the number to use as the devisor.
 * 
 * @return True if the integer is a multiple of the base number, otherwise false.
 */
bool CConvert::CheckInputDiv(size_t nLen, size_t nBase)
{
	const ldiv_t div_result = ldiv( static_cast<long>(nLen), static_cast<long>(nBase) );

	return 0 == div_result.rem;
}



/*@{
 * Wide strings are not universally available so we wrap them for conditional compilation
 * here.  As far as I can tell there's not a standard test we can use so this is a hack.
 * Feel free to improve upon.  The wish as that we don't have to invent a special pre-
 * processor symbol for our projects but instead base this check on ones already in use.
 *@}
 */
#if defined(HAS_WSTRING)

/** 
 * Converts a value into a wide string.
 *
 * @param nValue A size_t value to be converted.
 *
 * @return A std::wstring based on the input value.
 */
wstring CConvert::IntToStrW(size_t nValue)
{
	wostringstream o;

	o << nValue;

	return o.str();
}


/** 
 * Converts a value into a wide string.
 *
 * @param nValue An integer value to be converted.
 *
 * @return std::wstring based on the input value.
 */
wstring CConvert::IntToStrW(int nValue)
{
	wostringstream o;

	o << nValue;

	return o.str();
}


/** 
 * Converts a character into a wide string.
 *
 * @param cValue A char to be converted to a string.
 *
 * @return std::wstring based on the input value.
 */
wstring CConvert::CharToStrW( char cValue )
{
#if defined UNDER_CE
	int nResult = 0;
	wstring strOut;

	// get the lenght of the string
	int nLen = ::MultiByteToWideChar(CP_ACP, 0, &cValue, -1, NULL, NULL);

	// allocate space for the wide character string
	wchar_t* pcOut = new wchar_t[nLen+1];

    // Covert to Unicode.
	nResult = MultiByteToWideChar(CP_ACP, 0,  &cValue, -1, pcOut, nLen);	

	if (nResult == 0)
	{
		strOut = L"";
	}
	else
	{
		strOut = pcOut;
	}

	delete[] pcOut;

	return strOut;
#else
	wchar_t cValueW = 0;
	mbtowc( &cValueW, &cValue, sizeof(cValue));
	return wstring( 1, cValueW );
#endif
}


/** 
 * Converts an unsigned character into a wide string.
 *
 * @param cValue An unsigned char to be converted to a string.
 *
 * @return std::wstring based on the input value.
 */
wstring CConvert::UCharToStrW(unsigned char cValue)
{
	// Stream used for type conversion.
	wstringstream ss;

	// Pipe the value to a stream for type conversion
	ss << cValue;

	// Convert the stream to a string value
	return ss.str();
}


/**
 * This will convert a hexidecimal string in readable format to a vector of unsigned characters.
 *
 * @param pcIn - unsigned char* that is the pointer to the input data. 
 * @param nInLen - is the number of characters in the input data to convert.
 *
 * @return A hexidecial string representation of the binary data.
 */
std::wstring CConvert::BinToHexStringW(const unsigned char* pcIn, size_t nInLength)
{
	// get the string
	const string strReturnA = BinToHexStringA(pcIn, nInLength);

	// convert to wide character
	wstring strReturnW;
	CStringUtils::Widen( strReturnW, strReturnA );

	// return the string
	return strReturnW;
}


/**
 * This will convert a string that is that is in readable hex format into binary.
 *
 * @param str - const string& str that is the input hexidicimal string.
 * @param pcOut - unsigned char* that is the pointer in which to put the data. Note that this must be allocated
 *                prior to passing into this function.  The length sould be 1/2 of the string input length.
 * @param nOutLen - is the number of characters in the string.
 *
 * @note It will throw and error if the string is of 0 length.
 */
void CConvert::HexStringToBin(const std::wstring& str, unsigned char* pcOut, size_t& nOutLength)
{
	// narrow the input string
	string strA;
	CStringUtils::Narrow( strA, str );

	// get the binary data
	HexStringToBin( strA, pcOut, nOutLength );
}


wstring CConvert::IntToHexStringW(int nNum)
{
	string strNarrow = IntToHexStringA(nNum);
	wstring strWide = L"";

	CStringUtils::Widen(strWide, strNarrow);

	return strWide;
}


wstring CConvert::IntToHexStringW(unsigned int nNum)
{
	string strNarrow = IntToHexStringA(nNum);
	wstring strWide = L"";

	CStringUtils::Widen(strWide, strNarrow);

	return strWide;
}


wstring CConvert::ShortToHexStringW(short nNum)
{
	string strNarrow = ShortToHexStringA(nNum);
	wstring strWide = L"";

	CStringUtils::Widen(strWide, strNarrow);

	return strWide;
}


wstring CConvert::ShortToHexStringW(unsigned short nNum)
{
	string strNarrow = ShortToHexStringA(nNum);
	wstring strWide = L"";

	CStringUtils::Widen(strWide, strNarrow);

	return strWide;
}

#endif //test for wstring

