#pragma once

#include "CompileTimeCheck.h"
#include "boost/cstdint.hpp"
#include <stddef.h> //for size_t
#include <stdexcept> //for invalid_argument
#include <iterator>


namespace Field
{
	struct ThrowIfNull
	{
		inline static void Check( const void* )
		{
			throw std::invalid_argument( "Cannot read from NULL pointer" );
		}
	};

	struct NoCheck
	{
		inline static void Check( const void* )
		{
			//do nothing
		}
	};

	template<int nBytes>
	struct _LangType //_prefixed because it's used internally: don't use directly
	{
		//enum Result {}; //purposely left empty to trigger compile error in event that
		                //this general definition is used instead of one of the specializations
	};

	template<> //specialization for 1 byte
	struct _LangType<1>
	{
		typedef boost::uint8_t Result;
	};

	template<> //specialization for 2 byte
	struct _LangType<2>
	{
		typedef boost::uint16_t Result;
	};

	template<> //specialization for 4 byte
	struct _LangType<4>
	{
		typedef boost::uint32_t Result;
	};

	template<> //specialization for 8 byte
	struct _LangType<8>
	{
		typedef boost::uint64_t Result;
	};


	//Reads from a void* using type CFrom, casts & returns as type CTo
	template<typename CTo, typename CFrom>
	struct _Cast
	{
		inline static CTo Cast( const void* pFrom )
		{
			return static_cast<CTo>( *reinterpret_cast<const CFrom*>( pFrom ));
		}
	};


	//specialization for casting to a bool, since the compiler thinks it's more efficient
	//& gripes otherwise
	template<typename CFrom>
	struct _Cast<bool, CFrom>
	{
		inline static bool Cast( const void* pFrom )
		{
			return (0 != *reinterpret_cast<const CFrom*>( pFrom ));
		}
	};


	///Reads nBytes into to from pCur.  Checks pCur for NULL using NullCheck.
	///Returns nBytes.
	template<size_t nBytes, class NullCheck, typename CTo>
	inline size_t Read( CTo& to, const void* pCur )
	{
		NullCheck::Check( pCur );
		typedef typename _LangType<nBytes>::Result ReadType;
		STATIC_CHECK( sizeof(ReadType) == nBytes, Using_Incompatible_Fundamental_Type );
		//STATIC_CHECK( sizeof(CTo) <= nBytes, Loss_Of_Data_On_Cast );
		//use a template function here so we can specialize for CTo==bool
		to = _Cast<CTo, ReadType>::Cast( pCur );
		return nBytes;
	}


	///Reads sizeof(CTo) bytes into to from pCur.  Checks pCur for NULL using NullCheck.
	///Returns sizeof(CTo).
	template<class NullCheck, typename CTo>
	inline size_t Read( CTo& to, const void* pCur )
	{
		return Read<sizeof(CTo), NullCheck, CTo>( to, pCur );
	}


	///Returns nBytes from pCur.  Checks pCur for NULL using NullCheck.
	template<size_t nBytes, class NullCheck>
	inline typename _LangType<nBytes>::Result Read( const void* pCur )
	{
		NullCheck::Check( pCur );
		typedef typename _LangType<nBytes>::Result ReadType;
		STATIC_CHECK( sizeof(ReadType) == nBytes, Using_Incompatible_Fundamental_Type );
		return *reinterpret_cast<const ReadType*>( pCur );
	}


	///Returns CTo from pCur.  Checks pCur for NULL using NullCheck.
	template<typename CTo, class NullCheck>
	inline CTo Read( const void* pCur )
	{
		NullCheck::Check( pCur );
		return *reinterpret_cast<const CTo*>( pCur );
	}


	///Writes nBytes into pCur from from.  Checks pCur for NULL using NullCheck.
	///Returns nBytes.
	template<size_t nBytes, class NullCheck, typename CFrom>
	inline size_t Write( void* pCur, CFrom from )
	{
		NullCheck::Check( pCur );
		typedef typename _LangType<nBytes>::Result WriteType;
		STATIC_CHECK( sizeof(WriteType) == nBytes, Using_Incompatible_Fundamental_Type );
		*reinterpret_cast<WriteType*>( pCur ) = static_cast<WriteType>( from );
		return nBytes;
	}


	///Writes sizeof(CFrom) bytes into pCur from from.  Checks pCur for NULL using NullCheck.
	///Returns sizeof(CFrom).
	template<class NullCheck, typename CFrom>
	inline size_t Write( void* pCur, CFrom from )
	{
		return Write<sizeof(CFrom), NullCheck, CFrom>( pCur, from );
	}


	///Use w/ inserters.  Will NOT work w/ raw pointers or iterators: use Copy that
	///requires an "end" iterator.  (You get a compile error so there's no change of
	///accidental usage.)
	///
	///Designed to permit the writing of larger types into a container of smaller ones.
	///E.g., we can write:
	///	vector<char> vec;
	/// int n;
	///	Insert( back_inserter(vec), n );
	///
	///and expect the int to occupy 4 elements in the vector.  A compiler error is generated
	///if the container's data type does not evenly contain the type being stored.  This implies
	///that you can't stuff a smaller type into a container of larger types.
	///
	///\return Inserter to the next insertion point
	template<typename Inserter, typename T>
	inline Inserter Insert( Inserter iBuff, T tData )
	{
		typedef typename Inserter::container_type::value_type BuffType; //type the inserter iterates over
		enum { nBuffSize = sizeof tData / sizeof(BuffType) }; //# of elements in the container

		//require container to evenly store the source type
		//i.e., tData should evenly fit into 1 or more iBuff[]'s
		//e.g., int32 value fits into 4 chars
		STATIC_CHECK(
			nBuffSize * sizeof(BuffType) == sizeof tData,
			Container_Does_Not_Evenly_Hold_Type );

		//a semi-decent compiler will unroll this loop
		BuffType *pData = reinterpret_cast<BuffType*>( &tData );
		for( size_t i = 0; i < nBuffSize; ++i )
		{
			*iBuff++ = *pData++;
		}
		return iBuff;
	}


	///Meant for use w/ inserters
	///\return Next insertion point
	template<size_t nBytes, typename Inserter, typename T>
	inline Inserter Insert( Inserter iBuff, T tData )
	{
		typedef typename Inserter::container_type::value_type BuffType; //type the inserter iterates over
		enum { nBuffSize = nBytes / sizeof(BuffType) }; //# of elements in the container

		//require container to evenly store nBytes bytes
		//same test in other Insert, but w/ different error message
		STATIC_CHECK(
			nBuffSize * sizeof(BuffType) == nBytes,
			Container_Does_Not_Evenly_Hold_Requested_Byte_Count );

		return Insert( iBuff, static_cast<typename _LangType<nBytes>::Result>( tData ));
	}


	///Use with iterators or pointers
	template<typename Iterator, typename T>
	inline Iterator Copy( Iterator iBuff, Iterator iEnd, T tData )
	{
		typedef typename std::iterator_traits<Iterator>::value_type BuffType; //type the inserter iterates over
		enum { nBuffSize = sizeof tData / sizeof(BuffType) }; //# of elements in the container

		//require container to evenly store the source type
		//i.e., tData should evenly fit into 1 or more iBuff[]'s
		//e.g., int32 value fits into 4 chars
		STATIC_CHECK(
			nBuffSize * sizeof(BuffType) == sizeof tData,
			Container_Does_Not_Evenly_Hold_Type );

		//test that the output buffer is large enough to contain our type
		if( iEnd - iBuff < nBuffSize )
		{
			throw std::invalid_argument( "Buffer too short" );
		}

		//a semi-decent compiler will unroll this loop
		BuffType *pData = reinterpret_cast<BuffType*>( &tData );
		for( size_t i = 0; i < nBuffSize; ++i )
		{
			*iBuff++ = *pData++;
		}
		return iBuff;
	}


	///\return Next insertion point
	template<size_t nBytes, typename Iterator, typename T>
	inline Iterator Copy( Iterator iBuff, Iterator iEnd, T tData )
	{
		typedef typename std::iterator_traits<Iterator>::value_type BuffType; //type the inserter iterates over
		enum { nBuffSize = nBytes / sizeof(BuffType) }; //# of elements in the container

		//require container to evenly store nBytes bytes
		//same test in other Copy, but w/ different error message
		STATIC_CHECK(
			nBuffSize * sizeof(BuffType) == nBytes,
			Container_Does_Not_Evenly_Hold_Requested_Byte_Count );

		return Copy( iBuff, iEnd, static_cast<typename _LangType<nBytes>::Result>( tData ));
	}


	///Use with iterators or raw pointers.
	template<typename Iterator, typename T>
	inline Iterator Extract( T& tData, Iterator iBuff, Iterator iEnd )
	{
		typedef typename std::iterator_traits<Iterator>::value_type BuffType; //type the inserter iterates over
		enum { nBuffSize = sizeof tData / sizeof(BuffType) }; //# of elements in the container

		//require input buffer to evenly store the source type
		//i.e., tData should evenly fit into 1 or more iBuff[]'s
		//e.g., int32 value fits into 4 chars
		STATIC_CHECK(
			nBuffSize * sizeof(BuffType) == sizeof tData,
			Input_Does_Not_Evenly_Hold_Type );

		//test that the input buffer is large enough to contain our type
		if( iEnd - iBuff < nBuffSize )
		{
			throw std::invalid_argument( "Input too short" );
		}

		//a semi-decent compiler will unroll this loop
		BuffType *pData = reinterpret_cast<BuffType*>( &tData );
		for( size_t i = 0; i < nBuffSize; ++i )
		{
			*pData++ = *iBuff++;
		}
		return iBuff;
	}


} //namespace Field



//below is an in-progress experiment to make field handling as transparent as possible
/*
template<typename T, size_t _nBytes = sizeof(T)>
struct CField
{
	typedef T value_t;

	//constant size
	enum { nSize = _nBytes };

	explicit CField( const T& rhs = T()): m_value(rhs) {}

	template<typename R>
	CField( const CField<R>& rhs ): m_value(rhs.m_value) {}

	//keep default dtor, assignment & equality check
	//no virt dtor needed: this class is final

	//include for consistency
	size_t Size() const { return nSize; }

	//use w/ an inserter, not w/ an iterator or pointer
	template<typename Inserter>
	inline Inserter operator<<( Inserter iBuff ) const
	{
		return Field::Insert<nBytes>( iBuff, m_value );
	}

	//use w/ iterators or pointers
	template<typename Iterator>
	inline std::pair<Iterator, Iterator>& operator>>( std::pair<Iterator, Iterator>& iBuff )
	{
		iBuff.first = Field::Extract<nBytes>( m_value, iBuff.first, iBuff.second );
		return iBuff;
	}

	//getter
	const T& operator()() const { return m_value; }

	//conversion/getter
	operator const T&() const { return m_value; }

	//setter
	template<typename R>
	CField<T>& operator=( const R& rhs )
	{
		m_value = rhs;
		return *this;
	}
};
*/


