/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file FileUtils.cpp
 * Implementation file for the CFileUtils class which implements helper file
 * utility methods.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include "FileUtils.h"
#include "ApexException.h"
#include "StringUtils.h"
#include <string>
#include <windows.h>


///////////////////////////////////////////////////////////////////////////////
// undefines
///////////////////////////////////////////////////////////////////////////////

// Counteract Windows' wonderful macro name clobbering
#ifdef SetFileAttributes
#undef SetFileAttributes
#endif



///////////////////////////////////////////////////////////////////////////////
// defines
///////////////////////////////////////////////////////////////////////////////

#ifndef MAX_PATH
#define MAX_PATH 260
#endif


///////////////////////////////////////////////////////////////////////////////
// namespaces
///////////////////////////////////////////////////////////////////////////////

using namespace std;



/**
 * This method retrieves the path to the current executable. This will not
 * include the last \.
 * 
 * @return A std::wstring that contains the path to the current running executable.
 */
wstring CFileUtils::GetCurrentAppPath()
{
	try
	{
		wstring strFile;
		wstring strFolderPath;
		unsigned long nResult = 0;
		wchar_t cFileName[MAX_PATH+1] = {'\0'};

		// Get the path for the currently running app from the system.
		nResult = ::GetModuleFileNameW(NULL, cFileName, MAX_PATH);

		// Were we successful?
		if (nResult == 0)
		{
			// If not, then we need to raise an exception.
			wstring strError = L"Error getting the current application path.";

			// Get the error number for the exception.
			strError += CApexException::GetSystemError();

			// Throw the exception.
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		strFile = cFileName;
		size_t nPos = 0;

		// Get the position of the last slash in the path.
		nPos = strFile.rfind(L"\\");

		// Do we have a slash in the path?
		if (nPos == string::npos)
		{
			strFolderPath = L"\\";
		}
		else
		{
			// Remove the last slash in the path.
			strFolderPath = strFile.substr(0,nPos);
		}

		return strFolderPath;
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method deletes the file passed in.
*
* @param A const std::wstring holding the file that is to be deleted.
*/
void CFileUtils::FileDelete(const wstring& strFile)
{
	HANDLE hFile = INVALID_HANDLE_VALUE;

	try
	{
		// Check the length.
		if (strFile.length() < 1)
		{
			// Nothing was passed in so raise an exception.
			wstring strError = L"Error input file name is too short.";

			// Throw the exception.
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Make sure the file is writable.  If it is read only then an error would occur.
		SetFileWriteable(strFile);

		// Try to delete the file
		BOOL bDeleteResult = ::DeleteFile(strFile.c_str());

		// Were we successful?
		if (bDeleteResult != 0)
		{
			// File deleted successfully
			return;
		}

		// Error deleting file.  Get the error
		DWORD nError = ::GetLastError();

		// Did the file exist in the first place.
		if (nError == 2)
		{
			// No, so just return.
			return;
		}

		// Get the error that occured
		wstring strError = L"Error deleting file ";
		strError += strFile;
		strError += L". ";
		strError += CApexException::GetSystemError();

		// Throw the error
		throw CApexException( ERROR_FATAL, strError, __FILE__, __LINE__);
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown error.", __FILE__, __LINE__);
	}
}


/**
* This method sets the attributes of the file passed in to the attributes
* pass in as well.
*
* @param strFile A reference to a std::wstring that holds the filename of the file to be altered.
* @param nFileAttributes An unsigned long holding the file attributes to be applied to the file.
*
* @notes A single attribute or multiple attributes can be applied to the file
*        with a single call to the method. When specifying more than one
*        attribute, you should bitwise OR the attributes.
*
*        Such As: SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY);
*                 SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY | FILE_ATTRIBUTE_HIDDEN);
*
*        The file attributes available to you are:
*           FILE_ATTRIBUTE_ARCHIVE
*           FILE_ATTRIBUTE_HIDDEN
*           FILE_ATTRIBUTE_NORMAL
*           FILE_ATTRIBUTE_NOT_CONTENT_INDEXED
*           FILE_ATTRIBUTE_OFFLINE
*           FILE_ATTRIBUTE_READONLY
*           FILE_ATTRIBUTE_SYSTEM
*           FILE_ATTRIBUTE_TEMPORARY
*/
void CFileUtils::SetFileAttributes(const wstring& strFile, unsigned long nFileAttributes)
{
	HANDLE hFind = INVALID_HANDLE_VALUE;

	try
	{
		// make sure the file exists
		WIN32_FIND_DATAW FindFileData;
		hFind = ::FindFirstFile(strFile.c_str(), &FindFileData);
		
		// check if the file was found
		if (hFind != INVALID_HANDLE_VALUE)
		{
			BOOL bResult = FALSE;

			// try to set the file properties
			bResult = ::SetFileAttributesW(strFile.c_str(), nFileAttributes);
			if (bResult == 0)
			{
				// problem setting the file attributes
				wstring strError = L"Error setting file attributes for ";
				strError += strFile;
				strError += L". ";
				strError += CApexException::GetSystemError();
				::FindClose(hFind);
				throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
			}

			// delete the handle to the file
			if (hFind != INVALID_HANDLE_VALUE)
			{
				::FindClose(hFind);
				hFind = INVALID_HANDLE_VALUE;
			}
		}
	}
	catch( const CApexException& )
	{
		if (hFind != INVALID_HANDLE_VALUE)
		{
			::FindClose(hFind);
			hFind = INVALID_HANDLE_VALUE;
		}
		throw;
	}
	catch(...)
	{
		if (hFind != INVALID_HANDLE_VALUE)
		{
			::FindClose(hFind);
			hFind = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method makes sure the file passed in is writable.
*
* @param strFile A const std::wstring containing the filename to be made writeable.
*/
void CFileUtils::SetFileWriteable(const wstring& strFile)
{
	try
	{
		SetFileAttributes(strFile, FILE_ATTRIBUTE_NORMAL);
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method makes sure the file passed in is writable.
*
* @param strFile A const std::string containing the filename to be made writeable.
*/
void CFileUtils::SetFileWriteable(const string& strFile)
{
	try
	{
		wstring strFileW;

		// Convert the filename to a wide string.
		CStringUtils::Widen(strFileW, strFile);

		SetFileAttributes(strFileW, FILE_ATTRIBUTE_NORMAL);
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method makes sure the file passed in is read-only.
*
* @param strFile A const std::wstring containing the filename to be made read-only.
*/
void CFileUtils::SetFileReadonly(const wstring& strFile)
{
	try
	{
		SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY);
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method determines if a file exists or not. (Wide String Version)
*
* @param strFile A const std::wstring containing the file that is to be checked.
*
* @return A bool indicating true if the file exists, otherwise false.
*/
bool CFileUtils::FileExists(const wstring& strFile)
{
	HANDLE hFind = INVALID_HANDLE_VALUE;

	try
	{
		// Make sure the file exists
		WIN32_FIND_DATAW FindFileData;

		hFind = ::FindFirstFile(strFile.c_str(), &FindFileData);
		
		// Was the file found?
		if (hFind != INVALID_HANDLE_VALUE)
		{
			// Yes, so close any resources used.
			::FindClose(hFind);

			hFind = INVALID_HANDLE_VALUE;

			// Return indicating the file exists.
			return true;
		}

		// File does not exist.
		return false;
	}
	catch( const CApexException& )
	{
		// Delete the handle to the file
		if (hFind != INVALID_HANDLE_VALUE)
		{
			::FindClose(hFind);

			hFind = INVALID_HANDLE_VALUE;
		}

		throw;
	}
	catch(...)
	{
		// Delete the handle to the file
		if (hFind != INVALID_HANDLE_VALUE)
		{
			::FindClose(hFind);

			hFind = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method determines if a file exists or not.
*
* @param strFile A const std::string containing the file that is to be checked.
*
* @return A bool indicating true if the file exists, otherwise false.
*/
bool CFileUtils::FileExists(const string& strFile)
{
	try
	{
		// Return value from function.
		bool bResult = false;

		// String for the wide character file name.
		wstring strFileW;

		// Convert the input string to a wide string
		CStringUtils::Widen(strFileW, strFile);

		// See if the file exits
		bResult = FileExists(strFileW);

		// Return the result
		return bResult;
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}	
}


/**
* This method retrieves the size of the file (in bytes) that is passed in.
* (Wide String Version)
*
* @param strFile A const std::wstring holding the name of the file to check.
*
* @return An unsigned long value indicating the size of the file in bytes.
*/
unsigned long CFileUtils::FileGetSize(const wstring& strFile)
{
	HANDLE hFileIn  = NULL;
	DWORD nFileSize = 0;

	try
	{
		// Open the file to get the attributes
		hFileIn = CreateFile(strFile.c_str(),       
							0,                         // open for reading 
							FILE_SHARE_READ,           // share for reading 
							NULL,                      // default security
							OPEN_EXISTING,             // existing file only 
							FILE_ATTRIBUTE_NORMAL,     // normal file 
							NULL);                     // no attr. template 

		// Were we able to open the file?
		if (hFileIn == INVALID_HANDLE_VALUE) 
		{ 
			// No - there was a problem so we need to raise an exception.
			wstring strError = L"Error opening file to get attributes for ";

			// Add the filename to the message.
			strError += strFile;
			strError += L". ";

			// Add the error number to the message.
			strError += CApexException::GetSystemError();

			// Throw the exception.
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		} 

		// Get the input file size
		// Does not support files larger than 32 bits  (2 billion)
		nFileSize = ::GetFileSize(hFileIn,NULL);

		// Did we get a valid size?
		if (nFileSize == INVALID_FILE_SIZE)
		{
			// Unable to get file size so we neeed to throw and exception.
			wstring strError = L"Error getiting file size for ";

			// Set the filename in the error.
			strError += strFile;

			strError += L". ";

			// Set the error number in the error message.
			strError += CApexException::GetSystemError();
			
			// Close the file handle
			if (hFileIn != INVALID_HANDLE_VALUE)
			{
				::CloseHandle(hFileIn);

				hFileIn  = INVALID_HANDLE_VALUE;
			}

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Close the file handle
		if (hFileIn != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileIn);

			hFileIn  = INVALID_HANDLE_VALUE;
		}
		
		// Return the file size
		return nFileSize;
	}
	catch( const CApexException& )
	{
		// Close the handle if still open
		if (hFileIn != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileIn);

			hFileIn = INVALID_HANDLE_VALUE;
		}

		throw;
	}
	catch(...)
	{
		// Close the handle if still open
		if (hFileIn != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileIn);

			hFileIn = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method retrieves the size of the file (in bytes) that is passed in.
*
* @param strFile A const std::string holding the name of the file to check.
*
* @return An unsigned long value indicating the size of the file in bytes.
*/
unsigned long CFileUtils::FileGetSize(const string& strFile)
{
	DWORD nFileSize = 0;
	wstring strFileW;

	try
	{
		// Convert the file name to be unicode
		CStringUtils::Widen(strFileW, strFile);

		// Get the file size
		nFileSize = FileGetSize(strFileW);
				
		// Return the size
		return nFileSize;
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method reads data from the file passed in and returns the data in a
* vector of unsigned char(s). (Wide String Version)
*
* @param strFile A const std::wstring holding the name of the file that data will be read from.
* @param nMinReadSize A size_t that holds the minimum size of the file to be read.
* @param nMaxReadSize A size_t that holds the maximum size of the file to be read.
*
* @return A vector of unsigned char(s) that holds the data read from the file.
*/
vector<unsigned char> CFileUtils::FileRead(const wstring& strFile, size_t nMinReadSize, size_t nMaxReadSize)
{
	HANDLE hFile = INVALID_HANDLE_VALUE;
	vector<unsigned char> vecReturn;

	try
	{
		bool bResult = false;
		unsigned long nFileSize = 0;
		unsigned long nBytesRead = 0;
		
		// Check if the file exists
		bResult = FileExists(strFile);

		// Did we find the file?
		if (bResult == false)
		{
			// File does not exist, so we need to raise an exception.
			wstring strError = L"Error file ";

			// Set the filename into the error message.
			strError += strFile;

			strError += L" does not exists. ";
			
			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Get the size of the file
		nFileSize = FileGetSize(strFile);

		// Check if the file size is not larger than the maximum size
		if (nFileSize > nMaxReadSize)
		{
			// The file size is too big.
			wstring strError = L"Error file ";
			strError += strFile;
			strError += L" size is too large. ";
			
			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Check the minimum size
		if (nFileSize < nMinReadSize)
		{
			// The file size is too small.
			wstring strError = L"Error file ";
			strError += strFile;
			strError += L" size is too small. ";
			
			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Open the file for reading
		hFile = CreateFile(strFile.c_str(),    
							GENERIC_READ,             // open for writing 
							FILE_SHARE_READ,           // share for reading 
							NULL,                      // no security 
							OPEN_EXISTING,             // open only existing files 
							FILE_ATTRIBUTE_NORMAL,     // normal file 
							NULL);                     // no attr. template 

		// Were we able to open the file?
		if (hFile == INVALID_HANDLE_VALUE) 
		{ 
			// There was a problem so we need to raise an exception.
			wstring strError = L"Error opening file " + strFile + L". " + CApexException::GetSystemError();

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// File open successful - Resize the vector to take the data
		vecReturn.resize(nFileSize);

		// Read the bytes from the file
		BOOL bFileReadResult = ReadFile(hFile, &vecReturn[0] , nFileSize,	&nBytesRead, NULL);

		// Was the read successful?
		if (bFileReadResult == 0  || nBytesRead != nFileSize)
		{
			// Error occurred reading - we need to raise an exception.
			wstring strError = L"Error reading from file ";

			// Inserte the filename into the error message.
			strError += strFile;

			strError += L". ";

			// Get the error number for the error message.
			strError += CApexException::GetSystemError();
			
			// Close the file
			if (hFile == INVALID_HANDLE_VALUE) 
			{
				::CloseHandle(hFile);

				hFile = INVALID_HANDLE_VALUE;
			}

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}

		// Return the data
		return vecReturn;
	}
	catch( const CApexException& )
	{
		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}
		throw;
	}
	catch(...)
	{
		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method reads data from the file passed in and returns the data in a
* vector of unsigned char(s).
*
* @param strFile A const std::string holding the name of the file that data will be read from.
* @param nMinReadSize A size_t that holds the minimum size of the file to be read.
* @param nMaxReadSize A size_t that holds the maximum size of the file to be read.
*
* @return A vector of unsigned char(s) that holds the data read from the file.
*/
vector<unsigned char> CFileUtils::FileRead(const string& strFile, size_t nMinReadSize, size_t nMaxReadSize)
{
	vector<unsigned char> vecData;
	wstring strFileW;

	try
	{
		// Convert the file name to be unicode
		CStringUtils::Widen(strFileW, strFile);

		// Get the file size
		vecData = FileRead(strFileW);
				
		// Return the data
		return vecData;
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method writes data to the filename passed into it. (Wide String Version)
*
* @param strFile A const std::wstring that holds the filename to be written to.
* @param vecData A vector containing the data (unsigned char) to be written to the file.
* @param bOverwrite A boolean indicating whether we should overwrite a file if it already exists.
*
* @notes This method will only write data to the file if it does not already
*        exist or if the boolean parameter is set to true. If the file already
*        exists then an exception will be thrown.
*/
void CFileUtils::FileWrite(const wstring& strFile, vector<unsigned char> vecData, bool bOverwrite)
{
	HANDLE hFile = INVALID_HANDLE_VALUE;

	try
	{
		// Are we going to just overwrite any existing file?
		if (bOverwrite == true)
		{
			// If so, just delete the file.
			FileDelete(strFile);
		}
		else
		{
			// We are not just overwriting the file, so
			// check if the file exists already?
			bool bResult = FileExists(strFile);

			// Does the file exist?
			if (bResult == true)
			{
				// If so, set the system error to a sharing violation.
				::SetLastError(ERROR_SHARING_VIOLATION);

				// Now we need to raise an exception.
				wstring strError = L"The file ";
				strError += strFile;
				strError += L" already exists. ";
				strError += CApexException::GetSystemError();

				// Throw the exception
				throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
			}
		}

		// Create the file
		hFile = CreateFile(strFile.c_str(),    
							GENERIC_WRITE,             // open for writing 
							FILE_SHARE_READ,           // share for reading 
							NULL,                      // no security 
							OPEN_ALWAYS,               //  
							FILE_ATTRIBUTE_NORMAL,     // normal file 
							NULL);                     // no attr. template 

		// Were we able to create the file successfully?
		if (hFile == INVALID_HANDLE_VALUE) 
		{ 
			// We had a failure on the creation so we need to raise an exception.
			wstring strError = L"Error creating file ";
			strError += strFile;
			strError += L". ";
			strError += CApexException::GetSystemError();

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}


		unsigned long nBytesWritten = 0;
		BOOL bFileWriteResult = FALSE;

		// Write the data to the file.
		bFileWriteResult = ::WriteFile(hFile, &vecData[0], static_cast<DWORD>(vecData.size()), &nBytesWritten, NULL);

		// Were we successful in the write and did we write all of the data?
		if (bFileWriteResult == 0 || nBytesWritten != vecData.size())
		{
			// There was an issue with the write so we need to raise an exception.
			wstring strError = L"Error writing to file ";
			strError += strFile;
			strError += L". ";
			strError += CApexException::GetSystemError();
			
			// Close the file
			if (hFile == INVALID_HANDLE_VALUE) 
			{
				::CloseHandle(hFile);

				hFile = INVALID_HANDLE_VALUE;
			}

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}
	}
	catch( const CApexException& )
	{
		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}

		throw;
	}
	catch(...)
	{
		// Close the file handle
		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			hFile  = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method writes data to the filename passed into it.
*
* @param strFile A const std::string that holds the filename to be written to.
* @param vecData A vector containing the data (unsigned char) to be written to the file.
* @param bOverwrite A boolean indicating whether we should overwrite a file if it already exists.
*
* @notes This method will only write data to the file if it does not already
*        exist or if the boolean parameter is set to true. If the file already
*        exists then an exception will be thrown.
*/
void CFileUtils::FileWrite(const string& strFile, vector<unsigned char> vecData, bool bOverwrite)
{
	wstring strFileW;

	try
	{
		// Convert the file name to be unicode
		CStringUtils::Widen(strFileW, strFile);

		// Write the data to the file
		FileWrite(strFileW, vecData, bOverwrite);
	}
	catch( const CApexException& )
	{
		throw;
	}
	catch(...)
	{
		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method will compare two files and indicate if both the files are the same.
* (Wide String Version)
*
* @param strFileOne A const std::wstring that holds the filename of the first file to compare.
* @param strFileTwo A const std::wstring that holds the filename of the second file to compare.
* @param pnPos A pointer to a size_t which will return the byte position of where the files begin to differ.
*
* @return A boolean indicating if the two files are the same. True if they are the same, otherwise false.
*
* @notes If the files are the same the position pointer will point to the last byte in the file.
*/
bool CFileUtils::FileCompare(const wstring& strFileOne, const wstring& strFileTwo, size_t* pnPos)
{
	bool bReturn = true;
	size_t nReturnPos = wstring::npos;
	HANDLE hFileOne = INVALID_HANDLE_VALUE;
	HANDLE hFileTwo = INVALID_HANDLE_VALUE;
	wstring strTemp; // allow swapping of name
	size_t nTemp = 0;
	BOOL bResult = FALSE;  // return value for system calls
	size_t nLoop = 0; // loop for the file
	wstring strFileSmall;
	wstring strFileLarge;
	bool bCompare = true;

	try
	{
		// Check for valid input 
		if (strFileOne.length() < 1 )
		{
			// Raise an exception because the filename is not valid.
			wstring strError = L"Invalid input to FileCompare().  The first file name must have a length > 0.";

			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);		
		}

		// Check for valid input
		if (strFileTwo.length() < 1)
		{
			// Raise an exception because the filename is not valid.
			wstring strError = L"Invalid input to FileCompare().  The second file name must have a length > 0.";

			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);		
		}

		// Check for valid input
		if (pnPos == NULL)
		{
			// Raise an exception because the file position cannot be null.
			wstring strError = L"Invalid input to FileCompare().  The third argument cannot be NULL.";

			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);		
		}

		// Get the size of both input files
		size_t nSizeOne = FileGetSize(strFileOne);
		size_t nSizeTwo = FileGetSize(strFileTwo);

		// Arrange the files for easier understanding. Determine which file
		// is larger and which is smaller.
		if (nSizeOne > nSizeTwo)
		{
			strFileSmall = strFileTwo;
			strFileLarge = strFileOne;

			// Wwap the sizes
			nTemp = nSizeTwo;
			nSizeTwo = nSizeOne;
			nSizeOne = nTemp;
		}
		else
		{
			strFileSmall = strFileOne;
			strFileLarge = strFileTwo;
		}

		// Open the small file.
		hFileOne = CreateFile(strFileSmall.c_str(),    
							GENERIC_READ,             // open for writing 
							FILE_SHARE_READ,           // share for reading 
							NULL,                      // no security 
							OPEN_EXISTING,               //  
							FILE_ATTRIBUTE_NORMAL,     // normal file 
							NULL);                     // no attr. template 

		// Were we able to open the file?
		if (hFileOne == INVALID_HANDLE_VALUE) 
		{ 
			// Unable to open the file so we need to raise an exception.
			wstring strError = L"Error openign file ";
			strError += strFileSmall;
			strError += L". ";
			strError += CApexException::GetSystemError();

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		// Open the larger file.
		hFileTwo = CreateFile(strFileLarge.c_str(),    
							GENERIC_READ,             // open for writing 
							FILE_SHARE_READ,           // share for reading 
							NULL,                      // no security 
							OPEN_EXISTING,               //  
							FILE_ATTRIBUTE_NORMAL,     // normal file 
							NULL);                     // no attr. template 

		// Were we able to open the file?
		if (hFileTwo == INVALID_HANDLE_VALUE) 
		{ 
			// Unable to open the file so we need to raise an exception.
			wstring strError = L"Error opening file ";
			strError += strFileLarge;
			strError += L". ";
			strError += CApexException::GetSystemError();

			// Throw error
			throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
		}

		char cBufOne[8] = {'\0'};
		char cBufTwo[8] = {'\0'};
		unsigned long nBytesRead = 0;

		// Loop through the files and look for the first non matching character
		for (nLoop = 0; nLoop < nSizeOne; ++nLoop)
		{
			// read a byte from the first file
			bResult = ReadFile(hFileOne, &cBufOne, sizeof(cBufOne), &nBytesRead, NULL);

			// Were we successful in our read?
			if (bResult == 0 || nBytesRead != 1)
			{
				// Unable to read from the file, so we need to raise an exception.
				wstring strError = L"Error reading file ";
				strError += strFileLarge;
				strError += L". ";
				strError += CApexException::GetSystemError();

				// Throw error
				throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
			}

			// Reset the number of bytes read
			nBytesRead = 0;
			bResult = 0;

			// Read a byte from the second file
			bResult = ReadFile(hFileOne, &cBufTwo, sizeof(cBufTwo), &nBytesRead, NULL);
			
			// Were we successful in our read?
			if (bResult == 0 || nBytesRead != 1)
			{
				// Unable to read from the file, so we need to raise an exception.
				wstring strError = L"Error reading file ";
				strError += strFileLarge;
				strError += L". ";
				strError += CApexException::GetSystemError();

				// Throw error
				throw CApexException( ERROR_WARNING, strError, __FILE__, __LINE__);
			}

			// Are the bytes from each file different?
			if (cBufOne[0] != cBufTwo[0])
			{
				// If so, set our flag to indicate that the files are not the same.
				bReturn = false;

				// Break out of our compare loop.
				break;
			}
		}

		// Close the file handle for file one
		if (hFileOne != INVALID_HANDLE_VALUE)
		{
			bResult = ::CloseHandle(hFileOne);

			// Were we successful in closing the file?
			if (bResult == 0)
			{
				// If not, we need to raise an exception.
				wstring strError = L"Error closing file "  ;
				strError += strFileSmall;
				strError += CApexException::GetSystemError();
				throw CApexException( ERROR_FATAL, strError  , __FILE__, __LINE__);
			}

			hFileOne  = INVALID_HANDLE_VALUE;
		}

		// Close the file handle for file two
		if (hFileTwo != INVALID_HANDLE_VALUE)
		{
			bResult = ::CloseHandle(hFileTwo);

			// Were we successful in closing the file?
			if (bResult == 0)
			{
				// If not, we need to raise an exception.
				wstring strError = L"Error closing file "  ;
				strError += strFileLarge;
				strError += CApexException::GetSystemError();
				throw CApexException( ERROR_FATAL, strError  , __FILE__, __LINE__);
			}

			hFileTwo  = INVALID_HANDLE_VALUE;
		}

		// Do the sizes of the files match?
		if (nSizeOne != nSizeTwo)
		{
			// If not, then set our flag to indicate the files do not match.
			bReturn = false;
		}

		// Set the position
		*pnPos = nLoop;

		// Return
		return bReturn;

	}
	catch( const CApexException& )
	{
		// Close the file handle
		if (hFileOne != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileOne);

			hFileOne  = INVALID_HANDLE_VALUE;
		}

		// Close the second file handle
		if (hFileTwo != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileTwo);

			hFileTwo  = INVALID_HANDLE_VALUE;
		}

		throw;
	}
	catch(...)
	{
		// Close the file handle
		if (hFileOne != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileOne);

			hFileOne  = INVALID_HANDLE_VALUE;
		}

		// Close the second file handle
		if (hFileTwo != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFileTwo);

			hFileTwo  = INVALID_HANDLE_VALUE;
		}

		throw CApexException( ERROR_FATAL, L"Unknown exception.", __FILE__, __LINE__);
	}
}


/**
* This method will compare two files and indicate if both the files are the same.
*
* @param strFileOne A const std::string that holds the filename of the first file to compare.
* @param strFileTwo A const std::string that holds the filename of the second file to compare.
* @param pnPos A pointer to a size_t which will return the byte position of where the files begin to differ.
*
* @return A boolean indicating if the two files are the same. True if they are the same, otherwise false.
*
* @notes If the files are the same the position pointer will point to the last byte in the file.
*/
bool CFileUtils::FileCompare(const string& strFileOne, const string& strFileTwo, size_t* pnPos)
{
	wstring strFileOneW; 
	wstring strFileTwoW;
	bool bReturn = false;

	// Convert the file names to be in wide character
	CStringUtils::Widen(strFileOneW, strFileOne);
	CStringUtils::Widen(strFileTwoW, strFileTwo);

	// Call the compare wide character version
	bReturn = FileCompare(strFileOneW, strFileTwoW, pnPos);

	return bReturn;
}



