/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * Header file for CStringUtils class which is a utility class that contains
 * std::string methods that are commonly used.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include "Config.h"
#include <string>



/**
 * This class contains std::string functions that are commonly used.
 */
namespace CStringUtils
{
	typedef std::string::size_type size_type;

	extern const std::string strDefaultStrip; //" \t\r\n\v\f\0"

	/**
	* This will trim the left side of a string.
	*
	* @param strInput - This is the input string to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::string containing the original string with the left side stripped.
	*/
	std::string TrimLeft(const std::string& strInput, const std::string& strStrip = strDefaultStrip );

	/**
	* This will trim the left side of a string.
	*
	* @param strInput - This is the input string to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::string containing the original string with the right side stripped.
	*/
	std::string TrimRight(const std::string& strInput, const std::string& strStrip = strDefaultStrip );

	/**
	* This will trim the left and right side of a string.
	*
	* @param strInput - This is the input string to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::string containing the original string with the left and right
	*         sides stripped.
	*/
	std::string Trim(const std::string& strInput, const std::string& strStrip = strDefaultStrip );

	/**
	* This will trim the left and right side of a string.
	*
	* @param str String to be trimmed.
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*/
	void Trim( std::string& str, const std::string& strStrip = strDefaultStrip );

	/**
	* This method copies a C-string, stopping at '\0' or when the max length
	* has been reached.
	*
	* @param rhs A void pointer with the data to be copied.
	* @param nMaxStrLen A size_type indicating the number of characters to copy.
	*
	* @return A std:string with the copied data.
	*
	* @notes This method will copy all data until either a '\0' is reached
	*        or nMaxStrLen characters have been copied.
	*/
	std::string Copy( const void* rhs, size_type nMaxStrLen );

	/**
	* This method copies a C-string, stopping when a '/0' is reached or
	* at the ending location passed in.
	*
	* @param pvRhs A void pointer with the data to be copied.
	* @param pEnd A void pointer indicating the ending location of the data to be copied.
	*
	* @return A std:string with the copied data.
	*/
	std::string Copy( const void* rhs, const void* const pEnd );

	/**
	* This method right justifies a string, padding the left most characters
	* with the padding character specified.
	* 
	* @param strIn - The input string.
	* @param nWidth - The length of the string that we want returned.  
	* @param cPad - The pad character.  
	*
	* @return A std::string of size nWidth, with the rightmost (least significant)
	*         nWidth characters from strIn copied into the return value, padded
	*         with cPad on the left if necessary
	*/
	std::string RightJustify( const std::string& strIn, size_type nWidth, char cPad );

	/** 
	* This method left justifies a string, padding the right most characters
	* with the padding character specified.
	*
	* @param strIn - The input string.
	* @param nWidth - The length of the string that we want returned.  
	* @param cPad - The pad character.  
	*
	* @return A std::string of size nWidth, with the leftmost (most significant)
	*         nWidth characters from strIn copied into the return value, padded
	*         with cPad on the right if necessary
	*/
	std::string LeftJustify( const std::string& strIn, size_type nWidth, char cPad );

	/** 
	* Conver the string to lower case.
	* 
	* @param strIn - The string to convert.
	*
	* @return The lower case string.
	*/
	std::string  ToLower( const std::string& strIn);

	/** 
	* Conver the string to upper case.
	* 
	* @param strIn - The string to convert.
	*
	* @return The lower case string.
	*/
	std::string  ToUpper( const std::string& strIn);


/*@{
 * Wide strings are not universally available so we wrap them for conditional compilation
 *@}
 */
#if defined(HAS_WSTRING)

	extern const std::wstring wstrDefaultStrip; //" \t\r\n\v\f\0"

	/**
	* This will trim the left side of a string.
	*
	* @param strInput - This is the input wstring to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::wstring containing the original string with the left side
	*         stripped.
	*/
	std::wstring TrimLeft(const std::wstring& strInput, const std::wstring& strStrip = wstrDefaultStrip );

	/**
	* This will trim the left side of a wstring.
	*
	* @param strInput - This is the input wstring to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::wstring containing the original string with the right side
	*         stripped.
	*/
	std::wstring TrimRight(const std::wstring& strInput, const std::wstring& strStrip = wstrDefaultStrip );

	/**
	* This will trim the left side of a string.
	*
	* @param strInput - This is the input wstring to trim.  
	* @param strStrip - This is a parameter that describes the characters to strip.
	*                   By default if this parameter is not set it will be set to 
	*                   strDefaultStrip.  This will strip all white space.
	*
	* @return A std::wstring containing the original string with the left and
	*         right side stripped.
	*/
	std::wstring Trim(const std::wstring& strInput, const std::wstring& strStrip = wstrDefaultStrip );

	/**
	* Converts a narrow string to a wide string.
	*
	* @param strIn - the input narrow string.
	* @param strOut - the output wide string.
	*/
	void Widen( std::wstring& strOut, const std::string& strIn );

	/**
	* Converts a narrow string to a wide string.
	*
	* @param strOut - the output wide string.
	* @param strIn - the input narrow string.
	* @param nInLen - the lenght of the input narrow string.
	*/
	void Widen( std::wstring& strOut, const char* strIn, size_type nInLen );

	/**
	* Converts a wide string to a narrow string.
	*
	* @param strOut - the output narrow string.
	* @param strIn - the input wide string.
	*/
	void Narrow( std::string& strOut, const std::wstring& strIn );

	/**
	* Converts a wide string to a narrow string.
	*
	* @param strOut - the output narrow string.
	* @param strIn - the input wide string.
	* @param nInLen - the length of the input wide string.
	*/
	void Narrow( std::string& strOut, const wchar_t* strIn, size_type nInLen );

	/**
	* Converts a narrow string to a wide string.
	*
	* @param strIn - the input narrow string.
	*
	* @return A std::wstring with the data from the input narrow string
	*/
	std::wstring Widen( const std::string& strIn );

	/**
	* Converts a narrow string to a wide string.
	*
	* @param strIn - the input narrow string.
	* @param nInLen - the lenght of the input narrow string.
	*
	* @return A std::wstring with the data from the input narrow string
	*/
	std::wstring Widen( const char* strIn, size_type nInLen );

	/**
	* Converts a wide string to a narrow string.
	*
	* @param strIn - the input wide string.
	*
	* @return A std::string with the data from the input string
	*/
	std::string  Narrow( const std::wstring& strIn );

	/**
	* Converts a wide string to a narrow string.
	*
	* @param strIn - the input wide string.
	* @param nInLen - the length of the input string.
	*
	* @return A std::string with the data from the input string
	*/
	std::string  Narrow( const wchar_t* strIn, size_type nInLen );

	/** 
	* Conver the string to lower case.
	* 
	* @param strIn - The string to convert.
	*
	* @return The lower case string.
	*/
	std::wstring ToLower( const std::wstring& strIn);

	/** 
	* Conver the string to upper case.
	* 
	* @param strIn - The string to convert.
	*
	* @return The lower case string.
	*/
	std::wstring ToUpper( const std::wstring& strIn);

#endif //test for wstring
}


